####################################################
#
#	R Script for Random Effects meta-analysis of
#	binary data (Part 2): Meta-analysis
#	of Month 12 Mortality data
#
####################################################

#--- Load libraries ---#
library(metafor)	# This is the meta-analysis library in R

# Set working directory
setwd("INSERT YOUR DIRECTORY HERE")

#--------- Question 1. ---------#
#--- Read and print the data ---#
d <- read.csv("CochranePancreaticData12months.csv",na.strings=".", as.is=T)
print(d)

#--------- Question 2. ---------#
#--- Derive the log odds ratio and its variance ---#
d$p.G <- d$d.G / d$N.G
d$p.GC <- d$d.GC / d$N.GC

d$lor <- log( (d$p.GC/(1-d$p.GC)) / (d$p.G/(1-d$p.G)) )
d$var.lor <- 1/d$d.GC + 1/(d$N.GC-d$d.GC) + 1/d$d.G + 1/(d$N.G-d$d.G)


#-------- Question 3. ---------#
#--- Forest plot using log odds ratio ---#

ord <- order(d$lor)
forest(x=d$lor[ord], vi=d$var.lor[ord], slab=d$study[ord],
	ylim=c(-3,nrow(d)+3), refline=0,
	main="12 month mortality incidence\nGemcitabine+Chemotherapy vs. Gemcitabine")
text(-8, nrow(d)+1.5, "Study", cex=1, adj=0, font=2)
text(4.5, nrow(d)+1.5, "Est. + 95%CI",cex=0.95, adj=0, font=2)


#--- Forest plot on the odds ratio scale ---#
forest(x=d$lor[ord], vi=d$var.lor[ord], slab=d$study[ord],
	ylim=c(-3,nrow(d)+3), refline=0,
	atransf=exp,
	main="12 month mortality incidence\nGemcitabine+Chemotherapy vs. Gemcitabine")
text(-8, nrow(d)+1.5, "Study", cex=1, adj=0, font=2)
text(4.5, nrow(d)+1.5, "Est. + 95%CI",cex=0.95, adj=0, font=2)



#-------- Question 4. ---------#
#--- Fixed effects analysis of mortality incidence by hand ---#

w <- 1/d$var.lor
feEst <- sum(w*d$lor) / sum(w)
seFeEst <- sqrt(1/(sum(w)))
CIfe <- feEst + c(-1,0,1)*1.96*seFeEst

#-------- Question 5. ---------#
#--- Fixed effects analysis using rma function ---#
fema <- rma(yi=lor, vi=var.lor, data=d, method="FE",slab=study)
forest(fema, order='obs')

forest(fema, order='obs', atransf=exp)


#-------- Question 6. ---------#
#--- Fixed effects analysis using rma function and observed data ---#
fema2 <- rma(ai=d$d.GC, n1i=d$N.GC, ci=d$d.G, n2i=d$N.G, 
		data=d, method="FE", measure="OR")

print(fema)
