#####################################################
#
#	R code for setting-up fixed-effects meta-analysis
#	of LDL data for Atorvastatin 10 mg
#
#####################################################

# Load necessary libraries
library(lattice)
library(metafor)

# Set working directory to location where data resides
setwd('YOUR DIRECTORY GOES HERE')

# Read dataset
d <- read.csv('ldlDataAll.csv')

# Find studies that have atorvastatin
ator.list <- unique(d$trial[d$drug1=="Atorvastatin" & d$dose1>0])

# Find placebo studies (irrespective of other drugs)
pbo.list <- unique(d$trial[d$dose1==0 & d$dose2==0])

# Create dataset with placebo-contolled studies with Atorvastatin 
# (Note: not all of these are primarily atorvastatin studies)

ator <- d[d$trial %in% intersect(ator.list,pbo.list),]
ator <- ator[(ator$drug1=="Atorvastatin" & ator$dose2==0)|	# Atorvastatin mono tx
		 (ator$dose1==0 & ator$dose2==0),] 			# Placebo
ator$trial2 <- factor(ator$trial, levels=unique(as.character(ator$trial)))

# Summarize datasets
sink("LDL-all-summary.txt")
cat("Number of total trials =", length(unique(d$trial)), "\n")
cat("Drug combinations:\n")
print(table(d$drug1, d$drug2))
cat("Number of controlled Atorvastatin trials =", length(unique(ator$trial2)), "\n")
sink()

# Plot the atorvastatin data
colVec <- c("black","blue","red","orange")
png("AtorvastatinPlot.png")
xyplot(ldlPcfb~dose1, data=ator, groups=trial2, type='b', col=rep(colVec,2), 
	fill=rep(colVec,2),pch=21, cex=sqrt(ator$n/20),
	xlab="Atorvastatin Dose (mg)", ylab="Mean percent change from baseline LDL")
dev.off()

xyplot(ldlPcfb~dose1|trial, data=ator, type='b')

xyplot(ldlPcfb~dose1, data=ator, groups=trial, type='b', 
	subset=(dose1==0 | dose1==10))

xyplot(ldlPcfb~dose1|as.factor(week), data=ator, groups=trial, type='b')

# Output dataset
write.csv(ator, file="AtorvastatinStudies.csv", quote=F, row.names=F)

