####################################################
#
#	R Script for Random Effects meta-analysis of
#	binary data (Part 2): Meta-analysis
#	of Month 12 Mortality data
#
####################################################

#--- Load libraries ---#
library(metafor)	# This is the meta-analysis library in R

# Set working directory
setwd("INSERT YOUR DIRECTORY HERE")

#---- Data Preparation ----#
d <- read.csv("CochranePancreaticData12months.csv",na.strings=".", as.is=T)
print(d)
d$p.G <- d$d.G / d$N.G
d$p.GC <- d$d.GC / d$N.GC

d$lor <- log( (d$p.GC/(1-d$p.GC)) / (d$p.G/(1-d$p.G)) )
d$var.lor <- 1/d$d.GC + 1/(d$N.GC-d$d.GC) + 1/d$d.G + 1/(d$N.G-d$d.G)



#--------- Question 1. ---------#
#- Calculate Q and test H0:no heterogeneity -#
w <- 1/d$var.lor
feEst <- sum(w * d$lor) / sum(w)

k <- nrow(d)-1
Q <- sum( w*(d$lor - feEst)^2 )
pvalue.Q <- 1-pchisq(Q,k)

#--------- Question 2 ---------#
#- Calculate H and I2 -#
H <- sqrt(Q/k)
I2 <- 100*(Q-k)/Q

#--------- Question 3 ---------#
#- Calculate D-L estimate of between-study variance and RE est by hand -#
tau2.dl <- max( 0 , (Q - k) / ( sum(w) - sum(w^2)/sum(w) ) )
wre <- 1/(d$var.lor + tau2.dl)
reEst <- sum(wre * d$lor) / sum(wre)
varReEst <- 1/sum(wre)
reCI <- reEst + c(-1.96, 0, 1.96)*sqrt(varReEst)

# Compare the relative weights between fixed and random effects 
# models
print( w/sum(w) )
print( wre/sum(wre) )

#--------- Question 4 ---------#
#- Fit RE models using rma function -#
rema.dl <- rma(yi=lor, vi=var.lor, data=d, method="DL", slab=study)
confint(rema.dl)
coef(rema.dl)

rema.reml <- rma(yi=lor, vi=var.lor, data=d, method="REML", slab=study)
confint(rema.reml)
coef(rema.reml)

# Effect on the odds-ratio scale
exp(coef(rema.reml)[c(5,1,6)])


fema <- rma(yi=lor, vi=var.lor, data=d, method="FE", slab=study)


#-------- Question 5 ---------#
#--- Forest plot using log odds ratio

forest(rema.reml, refline=NA, order='obs',
	ylim=c(-3,nrow(d)+3), 
	mlab="RE model, REML estimate",
	main="12 month mortality incidence\nGemcitabine+Chemotherapy vs. Gemcitabine")
text(-8, nrow(d)+1.5, "Study", cex=1, adj=0, font=2)
text(4.5, nrow(d)+1.5, "Est. + 95%CI",cex=0.95, adj=0, font=2)
addpoly(rema.dl, col='red', mlab="RE model, D-L estimate", cex=1)
addpoly(fema, col='green', mlab="FE model", row=-3, cex=1)



