modelName <- "adasCog" # root names of model file

if(.Platform$OS.type == "windows") Sys.setenv(HOME = substr(R.home(), 1, 2))
courseDir <- file.path(Sys.getenv("HOME"), "MI260")
courseDir <- file.path(Sys.getenv("HOME"), "project/MI260W")
exampleDir <- file.path(courseDir, modelName)
toolsDir <- file.path(courseDir, "bugsTools") # change to match your setup

setwd(exampleDir)
library(lattice)
library(gdata)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))
extend.limits <- lattice:::extend.limits

## Get data file
adasData <- read.csv(file.path(exampleDir, "adasCogData.csv"), as.is = TRUE)

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"EDAPlots%03d.pdf", sep = "")),
	width = 6, height = 6, onefile = F)

for(drug in c("donepezil", "galantamine", "rivastigmine")) {

  studies <- unique(adasData$study)
  drug.in.study <- sapply(studies,
                          function(study, drug, xdata) {
                            drug %in% unique(xdata$DRGN[xdata$study == study])
                          },
                          drug = drug, xdata = adasData)
  studies <- studies[drug.in.study]
  xdata <- subset(adasData, (study %in% studies) & (DRGN %in% c(drug, "placebo")))
  xdata$treatment <- reorder.factor(as.factor(paste(xdata$DRGN, xdata$DTOT)))
  treatments <- levels(xdata$treatment)
  ntreat <- length(treatments)

  print(xyplot(ADAS ~ WEEK | as.factor(study),
               data = xdata, 
               groups = reorder.factor(as.factor(paste(DRGN, DTOT))), 
               panel = panel.superpose,
               panel.groups = function(x, y, group.number,
                 subscripts, ...) {
                 trt <- xdata$NMID[subscripts]
                 trts <- unique(trt)
                 for(itrt in trts) {
                   xx <- x[trt == itrt]
                   yy <- y[trt == itrt]
                   panel.xyplot(xx, yy, ...)
                 }
               },
               main = drug, 
               key = list(space = "right", type = "b", pch = 19, lwd = 2,
                 text = list(label = treatments, cex = 1.2),
                 lines = list(col = c("green", "red", "blue", "orange", "black", "magenta", "cyan", 
                                "purple", "grey", "brown", "light blue")[1:ntreat])),
               type = "b",
               lwd = 2,
               col = c("green", "red", "blue", "orange", "black", "magenta", "cyan", 
                 "purple", "grey", "brown", "light blue"),
               cex = 0.8,
               scales = list(cex=1.2, x = list(relation = "free")),
               xlab = list(label="time (w)", cex=1.2),
               ylab = list(label="mean ADAS-cog change from baseline", cex=1.2),
               par.strip.text = list(cex=1.2)
               )
        )

  print(xyplot(sd ~ WEEK | as.factor(study),
               data = xdata, 
               groups = reorder.factor(as.factor(paste(DRGN, DTOT))), 
               panel = panel.superpose,
               panel.groups = function(x, y, group.number,
                 subscripts, ...) {
                 trt <- xdata$NMID[subscripts]
                 trts <- unique(trt)
                 for(itrt in trts) {
                   xx <- x[trt == itrt]
                   yy <- y[trt == itrt]
                   panel.xyplot(xx, yy, ...)
                 }
               },
               main = drug, 
               key = list(space = "right", type = "b", pch = 19, lwd = 2,
                 text = list(label = treatments, cex = 1.2),
                 lines = list(col = c("green", "red", "blue", "orange", "black", "magenta", "cyan", 
                                "purple", "grey", "brown", "light blue")[1:ntreat])),
               type = "b",
               lwd = 2,
               col = c("green", "red", "blue", "orange", "black", "magenta", "cyan", 
                 "purple", "grey", "brown", "light blue"),
               cex = 0.8,
               scales = list(cex=1.2, x = list(relation = "free")),
               xlab = list(label="time (w)", cex=1.2),
               ylab = list(label="SD ADAS-cog change from baseline", cex=1.2),
               par.strip.text = list(cex=1.2)
               )
        )
    
}                                       # end loop over drugs
  
dev.off()
