modelName <- "adasCogHandsOn" # root names of model file

if(.Platform$OS.type == "windows") Sys.setenv(HOME = substr(R.home(), 1, 2))
courseDir <- file.path(Sys.getenv("HOME"), "MI260")
##courseDir <- file.path(Sys.getenv("HOME"), "project/MI260Summer2011")
exampleDir <- file.path(courseDir, modelName)
toolsDir <- file.path(courseDir, "bugsTools") # change to match your setup

setwd(exampleDir)
library(lattice)
library(gdata)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))
extend.limits <- lattice:::extend.limits

## Get data file
adasData <- read.csv(file.path(exampleDir, "donepezilAdasData.csv"), as.is = TRUE)

adasData$treatment <- reorder.factor(as.factor(paste(adasData$DRGN, adasData$DTOT)))
treatments <- levels(adasData$treatment)
ntreat <- length(treatments)

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"EDAPlots%03d.pdf", sep = "")),
	width = 6, height = 6, onefile = F)

  print(xyplot(ADAS ~ WEEK | as.factor(study),
               data = adasData, 
               groups = treatment, 
               panel = panel.superpose,
               panel.groups = function(x, y, group.number,
                 subscripts, ...) {
                 trt <- adasData$NMID[subscripts]
                 trts <- unique(trt)
                 for(itrt in trts) {
                   xx <- x[trt == itrt]
                   yy <- y[trt == itrt]
                   panel.xyplot(xx, yy, ...)
                 }
               },
               key = list(space = "right", type = "b", pch = 19, lwd = 2,
                 text = list(label = treatments, cex = 1.2),
                 lines = list(col = c("green", "red", "blue", "orange", "black", "magenta", "cyan", 
                                "purple", "grey", "brown", "light blue")[1:ntreat])),
               type = "b",
               lwd = 2,
               col = c("green", "red", "blue", "orange", "black", "magenta", "cyan", 
                 "purple", "grey", "brown", "light blue"),
               cex = 0.8,
               scales = list(cex=1.2, x = list(relation = "free")),
               xlab = list(label="time (w)", cex=1.2),
               ylab = list(label="mean ADAS-cog change from baseline", cex=1.2),
               par.strip.text = list(cex=1.2)
               )
        )

  print(xyplot(sd ~ WEEK | as.factor(study),
               data = adasData, 
               groups = treatment, 
               panel = panel.superpose,
               panel.groups = function(x, y, group.number,
                 subscripts, ...) {
                 trt <- adasData$NMID[subscripts]
                 trts <- unique(trt)
                 for(itrt in trts) {
                   xx <- x[trt == itrt]
                   yy <- y[trt == itrt]
                   panel.xyplot(xx, yy, ...)
                 }
               },
               key = list(space = "right", type = "b", pch = 19, lwd = 2,
                 text = list(label = treatments, cex = 1.2),
                 lines = list(col = c("green", "red", "blue", "orange", "black", "magenta", "cyan", 
                                "purple", "grey", "brown", "light blue")[1:ntreat])),
               type = "b",
               lwd = 2,
               col = c("green", "red", "blue", "orange", "black", "magenta", "cyan", 
                 "purple", "grey", "brown", "light blue"),
               cex = 0.8,
               scales = list(cex=1.2, x = list(relation = "free")),
               xlab = list(label="time (w)", cex=1.2),
               ylab = list(label="SD ADAS-cog change from baseline", cex=1.2),
               par.strip.text = list(cex=1.2)
               )
        )
      
dev.off()
