mbmaPpcPlot <- function(data, x = "x", treatment = "treatment", type = "type", xlab = "x"){

  names(data)[names(data) == x] <- "x"
  names(data)[names(data) == treatment] <- "treatment"
  names(data)[names(data) == type] <- "type"
  xobs <- data[data$type == "observed",]

  ## Reorder assuming that the predicted values are in the same order as the observed values.
  xOrder <- order(xobs$treatment, xobs$x)
  xOrder <- rep(xOrder, 4) + rep(0:3, ea = length(xOrder)) * length(xOrder)
  data <- data[xOrder,]
  xobs <- data[data$type == "observed",]

  ## Calculate number of treatments and number of arms of each treatment
  treats <- unique(xobs$treatment)
  nTreat <- length(treats)
  nPerTreat <- sapply(treats, function(treat, data)
                      sum(data$treatment == treat),
                      data = xobs)
  
  ## Calculate number of equally spaced positions on the y axis
  ny <- sum(nPerTreat) + nTreat - 1
  dy <- 1 / (ny - 1)
  y <- (1:ny) * dy

  ## Drop y positions between treatments to create gaps
  iSkip <- cumsum(nPerTreat[-nTreat]) + 1:(nTreat - 1)
  y <- y[-iSkip]

  ## Calculate label locations
  iStart <- c(1, 1 + cumsum(nPerTreat[-nTreat]))
  iEnd <- cumsum(nPerTreat)
  yLab <- (y[iStart] + y[iEnd]) / 2

  data$y <- rep(y, 4)
  xyplot(y ~ x, data, groups = type, panel =
         function(x, y, subscripts, groups, ...){
           iObs <- (groups[subscripts] == "observed")
           iMed <- (groups[subscripts] == "median")
           i5 <- (groups[subscripts] == "5%ile")
           i95 <- (groups[subscripts] == "95%ile")
           panel.xyplot(x[iObs], y[iObs], pch = 19, col = "black", ...)
           panel.segments(x[iMed], y[iMed] - dy/2, x[iMed], y[iMed] + dy/2,
                          col = "red", lwd = 2, ...)
           panel.segments(x[i5], y[iMed], x[i95], y[iMed],
                          col = "blue", lwd = 2, ...)
         },
         scales = list(cex = 1.2, y = list(at = yLab, labels = treats)),
         xlab = xlab, ylab = "")
}
