#####################################################
#
#	Homework 1 solutions
#
#####################################################

#--- Load libraries ---#
library(metafor)	# This is the meta-analysis library in R
library(R2WinBUGS)
library(emdbook)

#--- Set working directory ---#
#setwd("DIRECTORY WHERE YOU'RE KEEPING YOUR FILES")
setwd("E:/MI260USB/MI260/FixedEffectsHandsOn")
#setwd('~/Projects/svn-proj-MI-MBMAWeb/FixedEffectsHandsOn/')

#-----------------#
#--- Problem 1 ---#
#-----------------#

# 1. Read the data, derive the mean difference from placebo, and 
# corresponding standard error. 

d <- read.csv('AtorvastatinStudies.csv')
ator <- d[d$dose1>0,]
pbo <- d[d$dose1==0,c("trial","n","ldlPcfb","ldl0",
		"seLdl0","seLdlPcfb")]
names(pbo) <- paste(names(pbo),".pbo",sep='')

ator <- merge(ator, pbo, by.x="trial",by.y="trial.pbo", all=T)
ator$diff <- ator$ldlPcfb - ator$ldlPcfb.pbo
ator$seDiff <- sqrt(ator$seLdlPcfb^2 + ator$seLdlPcfb.pbo^2)

ator10 <- ator[ator$dose1==80,]

# 2. Make a forest plot of the data. Explore various options to make 
# your plot look the way you’d like. 
# Looking at the forest plot, do you think there is substantial 
# beween study variability in the difference from placebo?

# Table of data (compare to slides)
ator10 <- ator10[(order(ator10$diff)),]

with(ator10,
     forest(x=diff, sei=seDiff, 
            slab=trial, refline=NA, 
            at=seq(-80,-10,by=10),
            ylim=c(0,11),
#            xlim=c(-120,40),
            xlab="Mean Difference",
            main="Difference in %CFB LDL for 10 mg Atorvastatin")
)
text(-90.5, 9.5, "Study", cex=0.95, adj=0, font=2)
text(-15, 9.5, "Mean Diff. + 95%CI",cex=0.95, adj=0, font=2)

# 3. Calculate the fixed effects estimate using the rma function. 
# How can you see what the weights are for each of these studies? 
# Are there any studies that are having a large effect on the estimated 
# mean difference from placebo. What is the estimate of Q? 
# Would you conlcude that there is statistically significant heterogeneity?

fit1 <- rma(yi=diff, sei=seDiff, data=ator10, method="FE", slab=trial)
print(fit1)
names(fit1)
# Weights can be extracted with the weights function
weights(fit1)
# ... or relative weights
weights(fit1)/sum(weights(fit1))
# Q statistic and p-value
print(fit1)
#.. or ..
with(fit1, c(Q=QE,p.value=QEp))

# 4. Calculate the random effects estimate using the rma function. 
# Try using both the DerSimonian and Laird (method="DL") and 
# restricted maximum likelihood (method="REML") estimators of τ2. 
# How do the estimates compare?
fit2 <- rma(yi=diff, sei=seDiff, data=ator10, method="DL", slab=trial)
fit3 <- rma(yi=diff, sei=seDiff, data=ator10, method="REML", slab=trial)

# How to the estimates compare?
c(DL=fit2$tau2, REML=fit3$tau2)
sqrt(c(DL=fit2$tau2, REML=fit3$tau2))
# ... these look a little different, but what about uncertainty in the estimates?
confint(fit2, control=list(tau2.max=500))
confint(fit3, control=list(tau2.max=500))

# How to the weights compare?
feWeights <- weights(fit1)/sum(weights(fit1))
reWeights <- weights(fit3)/sum(weights(fit3))
mod <- factor(c(rep(1,6),rep(2,6)),levels=1:2,
              labels=c("FE model","RE model"))
xyplot(c(feWeights,reWeights) ~ mod,
       groups=rep(1:6,times=2), 
       col='black',pch=19,type='b',
       panel=function(x,y,groups,...) {
         panel.superpose(x,y,groups,...)
         panel.abline(h=1/6,col='red')
       },
       xlab='', ylab="Meta-analysis weight")

# 5. Look at the estimated values for Q and I2. What do you conclude 
# about heterogeneity of effect? Do you prefer the fixed or random 
# effects model?

print(fit3)

# ... Yes, heterogeneity.  I prefer the RE model to the FE model.

# 6. Using the forest and addpoly functions, make a forest plot 
# overlaying the data, the fixed effect estimate and a random effects 
# estimate of the mean difference from placebo.
forest(fit1, refline=NULL, 
       at=seq(-80,-10,by=10),
       ylim=c(-2.5,nrow(ator10)+3),  
       xlab="Mean Difference",
       main="Difference in %CFB LDL for 80 mg Atorvastatin")
text(-90.5, nrow(ator10)+1.5, "Study", cex=0.95, adj=0, font=2)
text(-15, nrow(ator10)+1.5, "Mean Diff. + 95%CI",cex=0.95, adj=0, font=2)
# Adding additional estimates
addpoly(fit3, row=-2, col='red', mlab='Random Effects model')

# 7. EXTRA: If you’re comfortable with Bayesian modeling, fit the 
# random effects model using WinBUGS and your choice of prior 
# distributions for θ and τ2. How do your results compare to the 
# results you obtained using rma?

mymodel <- function() {
  for (i in 1:nStudies) {
    thetahat[i] ~ dnorm(theta[i], precThetaHat[i])
    theta[i] ~ dnorm(meanDiff, prec)
    precThetaHat[i] <- pow(s[i],-2)
  }
  meanDiff ~ dnorm(0,1E-04)
  tau ~ dunif(0,100)
  prec <- pow(tau,-2)
}

bugsdata <- with(ator10, 
                 list(thetahat=diff, s=seDiff, 
                      nStudies=length(trial))
                 )

bugsParameters <- c("meanDiff","tau","theta")

bugsInits <- function() {
  return(list(meanDiff=rnorm(1, -20, 20), tau=runif(1,0,5))) 
}

reBugs1 <- bugs(data=bugsdata, inits=bugsInits, 
               parameters.to.save=bugsParameters,
               model=mymodel,
               n.chains=3, n.iter=10000, n.burnin=2000, n.thin=1,
               debug=TRUE)

# look at convergence diagnostics
plot(reBugs1)
print(reBugs1)
# convert bugs output to coda format
reBugscoda <- as.mcmc.bugs(reBugs1)
plot(reBugscoda)

# Add Bayes estimate to plot
forest(fit3, refline=NULL, 
       at=seq(-80,-10,by=10),
       ylim=c(-2.5,nrow(ator10)+3),  
       xlab="Mean Difference",
       main="Difference in %CFB LDL for 10 mg Atorvastatin",
       mlab="Classical RE model")
text(-90.5, nrow(ator10)+1.5, "Study", cex=0.95, adj=0, font=2)
text(-15, nrow(ator10)+1.5, "Mean Diff. + 95%CI",cex=0.95, adj=0, font=2)
# Adding additional estimates
addpoly(x=mean(reBugs1$sims.list$meanDiff), 
        vi=var(reBugs1$sims.list$meanDiff),
        row=-2, col='red', mlab='Bayesian RE model')


#-----------------#
#--- Problem 2 ---#
#-----------------#

# 1. Read the data, derive the log odds ratio 
# comparing GC to G and corresponding standard error.

d <- read.csv('CochranePancreaticData12months.csv')
d$logOR <- with(d, log(d.GC*(N.G-d.G)/(d.G*(N.GC-d.GC))))
d$varLogOR <- with(d, 1/d.GC + 1/(N.GC-d.GC) + 1/d.G + 1/(N.G-d.G))


# 2. Make a forest plot of the log odds ratio values. 
# Explore the effects of the transf and atransf options.
# Looking at the forest plot, do you think there is f
# substantial beween study variability?
d <- d[order(d$logOR),]
with(d,
     forest(x=logOR, vi=varLogOR, 
            slab=study, 
            atransf=exp,
            at=log(c(0.1,0.25,0.5,1,4,16)),
            alim=log(c(0.03,24)),
            refline=0, 
            #  alim=NULL,
         #   transf=exp,
         #   at=c(0.1,0.25,0.5,1,4,8),
         #   alim=c(0.03,8),
         #   refline=1,
            xlab="Odds Ratio",
            main="Odds Ratio for 12 month mortality\nGemcitabine+Chemo to Chemo")
)
text(log(0.0005), nrow(d)+1.5, "Study", cex=1, adj=0, font=2)
text(log(20), nrow(d)+1.5, "Odds Ratio [95%CI]",cex=1, adj=0, font=2)


# 3. Calculate the random effects estimate using the 
# rma function. What do you conclude about 
# heterogeneity of effect? Based on your answer, 
# would you prefer a fixed or random effects model?

fit.re <- rma(logOR,vi=varLogOR, data=d, method="REML", slab=study)
fit.fe <- rma(logOR,vi=varLogOR, data=d, method="FE", slab=study)

# 4. Using the forest and addpoly functions, make a 
# forest plot overlaying the data, the fixed effect 
# estimate and a random effects estimate of the odds 
# ratio.
forest(fit.re, atransf=exp,
       at=log(c(0.1,0.25,0.5,1,4,16)),
       alim=log(c(0.03,24)),
       refline=0, 
       ylim=c(-2.5,nrow(d)+3),
       xlab="Odds Ratio",
       main="Odds Ratio for 12 month mortality\nGemcitabine+Chemo to Chemo"
       )
text(log(0.0005), nrow(d)+1.5, "Study", cex=1, adj=0, font=2)
text(log(20), nrow(d)+1.5, "Odds Ratio [95%CI]",cex=1, adj=0, font=2)
addpoly(fit.fe, mlab="FE model", row=-2, col='red',
        atransf=exp, cex=1)

# 5. EXTRA: If you’re comfortable with Bayesian 
# modeling, fit the random effects model using WinBUGS 
# and your choice of prior distributions for θ and τ2. 
# How do your results compare to the results you 
# obtained using rma?

mymodel <- function() {
  for (i in 1:nStudies) {
    thetahat[i] ~ dnorm(theta[i], precThetaHat[i])
    theta[i] ~ dnorm(meanDiff, prec)
    precThetaHat[i] <- pow(s[i],-2)
  }
  meanDiff ~ dnorm(0,1E-04)
  tau ~ dunif(0,100)
  prec <- pow(tau,-2)
}

bugsdata <- list(thetahat=d$logOR, s=sqrt(d$varLogOR), 
                      nStudies=length(d$study))

bugsParameters <- c("meanDiff","tau","theta")

bugsInits <- function() {
  return(list(meanDiff=rnorm(1, 0, 5), tau=runif(1,0,5))) 
}

reBugs1 <- bugs(data=bugsdata, inits=bugsInits, 
                parameters.to.save=bugsParameters,
                model=mymodel,
                n.chains=3, n.iter=10000, n.burnin=2000, n.thin=1,
                debug=TRUE)

plot(reBugs1)
print(reBugs1)
#... probably want to run these chains for much longer
# to get a better posterior sample for tau

# convert bugs output to coda format
reBugscoda <- as.mcmc.bugs(reBugs1)
plot(reBugscoda)
traceplot(reBugscoda[,1], main=colnames(reBugscoda[[1]])[1])
traceplot(reBugscoda[,2], main=colnames(reBugscoda[[1]])[2])
traceplot(reBugscoda[[1]][,2], main=colnames(reBugscoda[[1]])[2])


# Add Bayes estimate to plot
forest(fit.re, atransf=exp,
       at=log(c(0.1,0.25,0.5,1,4,16)),
       alim=log(c(0.03,24)),
       refline=0, 
       ylim=c(-2.5,nrow(d)+3),
       xlab="Odds Ratio",
       main="Odds Ratio for 12 month mortality\nGemcitabine+Chemo to Chemo"
)
text(log(0.0005), nrow(d)+1.5, "Study", cex=1, adj=0, font=2)
text(log(20), nrow(d)+1.5, "Odds Ratio [95%CI]",cex=1, adj=0, font=2)
# Adding additional estimates
addpoly(x=mean(reBugs1$sims.list$meanDiff), 
        vi=var(reBugs1$sims.list$meanDiff),
        row=-2, col='red', mlab='Bayesian RE model',
        atransf=exp, cex=1)
