#####################################################
#
#	Homework 2 solutions
#
#####################################################

#--- Load libraries ---#
library(metafor)	# This is the meta-analysis library in R
library(R2WinBUGS)
library(emdbook)

#--- Set working directory ---#
#setwd("DIRECTORY WHERE YOU'RE KEEPING YOUR FILES")
setwd("E:/MI260USB/MI260/FixedEffectsHandsOn")

#-----------------#
#--- Problem 1 ---#
#-----------------#

# 1. Read the data, derive the mean difference from placebo, and 
# corresponding standard error. 

d <- read.csv('AtorvastatinStudies.csv')
ator <- d[d$dose1>0,]
pbo <- d[d$dose1==0,c("trial","n","ldlPcfb","ldl0",
		"seLdl0","seLdlPcfb")]
names(pbo) <- paste(names(pbo),".pbo",sep='')

ator <- merge(ator, pbo, by.x="trial",by.y="trial.pbo", all=T)
ator$diff <- ator$ldlPcfb - ator$ldlPcfb.pbo
ator$seDiff <- sqrt(ator$seLdlPcfb^2 + ator$seLdlPcfb.pbo^2)

ator80 <- ator[ator$dose1==80,]

# 2. Using the functions funnel and radial in the rma package, 
# make funnel and radial plots of the difference from placebo. 
# Do the plots look symmetric to you? Do you think there is any 
# evidence of small study bias?

fit1 <- rma(yi=diff, sei=seDiff, data=ator80, method="FE", slab=trial)
fit2 <- rma(yi=diff, sei=seDiff, data=ator80, method="REML", slab=trial)

# Let's see how these compare...
funnel(fit1, main="Funnel from fit1")
funnel(fit2,, main="Funnel from fit2")

# Code for radial plots.  Compare teh effect of the center option.
radial(fit1,digits=0,center=FALSE)
radial(fit1,digits=0,center=TRUE)


# 3. Plot the observed mean difference from placebo against the baseline 
# LDL using a scatterplot. Make a forest plot of the difference from 
# placebo, adding the baseline LDL as data (using the ilab option). 
# Which plot do you prefer? Does this variable seem promising to explain 
# some of the heterogeneity? Try making the same plots using the change 
# from baseline in the placebo group.

# Baseline LDL scatter plot
with(ator80, plot(ldl0, diff, cex=3/seDiff,pch=19,
                  xlab="Baseline mean LDL", 
                  ylab="Difference from placebo")
     )

# Forest plot with baseline LDL added
ord <- order(ator80$ldl0)
ator80 <- ator80[ord,]
forest(x=ator80$diff, vi=ator80$seDiff^2, 
       slab=ator80$trial, refline=NA,
       ilab=signif(ator80$ldl0,3),
       ilab.xpos=-90,
       xlim=c(-150,50),
#       alim=c(-90,0),
       ylim=c(0,nrow(ator80)+3),
       main="Difference from Placebo in %CFB LDL",
       cex=0.7)
text(-150, 7.5, "Study", cex=0.95, adj=0, font=2)
text(-95, 7.75, "Baseline\nLDL", cex=0.95, adj=0, font=2)
text(-30, 7.5, "Mean diff. + 95%CI",cex=0.95, adj=0, font=2)

# Placebo change from baseline scatter plot
with(ator80, plot(ldlPcfb.pbo, diff, cex=3/seDiff, pch=19,
                  xlab="Placebo change from baseline LDL",
                  ylab="Difference from placebo")
     )

# Forest plot 
ord <- order(ator80$ldlPcfb.pbo)
ator80 <- ator80[ord,]
forest(x=ator80$diff, vi=ator80$seDiff^2, 
       slab=ator80$trial, refline=NA,
       ilab=signif(ator80$ldlPcfb.pbo,3),
       ilab.xpos=-90,
       xlim=c(-150,50),
       #       alim=c(-90,0),
       ylim=c(0,nrow(ator80)+3),
       main="Difference from Placebo in %CFB LDL",
       cex=0.7)
text(-150, 7.5, "Study", cex=0.95, adj=0, font=2)
text(-95, 7.75, "Placebo\nresponse", cex=0.95, adj=0, font=2)
text(-30, 7.5, "Mean diff. + 95%CI",cex=0.95, adj=0, font=2)



# 4.Fit random-effects meta-regression model using both covariates. 
# First fit the models with one covariate, then fit a model with both 
# covariates. How would you interpret the coefficients in this meta-regression? 
# Is there a statistically significant effect of either or both covariates? 
# Is the estimated effect clinically meaningful? What might we conclude about
# the impact of the covariates on the drug effect? Is there additional 
# residual heterogeneity after including the covariates?

# Covariate=baseline LDL
fit3 <- rma(yi=diff, sei=seDiff, data=ator80, method="REML", slab=trial,
            mods=ldl0)
# ... with centered covariate
fit3 <- rma(yi=diff, sei=seDiff, data=ator80, method="REML", slab=trial,
            mods=ldl0-mean(ldl0))

# Covariate=placebo response
fit4 <- rma(yi=diff, sei=seDiff, data=ator80, method="REML", slab=trial,
            mods=ldlPcfb.pbo)
# ... with centered covariate
fit4 <- rma(yi=diff, sei=seDiff, data=ator80, method="REML", slab=trial,
            mods=ldlPcfb.pbo-mean(ldlPcfb.pbo))

# Covariates= baseline LDL & placebo response
fit5 <- rma(yi=diff, sei=seDiff, data=ator80, method="REML", slab=trial,
            mods=cbind(ldl0,ldlPcfb.pbo))

# 5.Plot the results from one or more of your models using a forest plot. 
# Add prediction intervals for placebo responses of -4%, -2%, 0%, and 2%.

# Get predictions
predVals <- predict(fit4, newmods=c(-2,0,2,4))

# Make plot
forest(fit4, refline=NA, #order='obs',
       cex=0.8, order='fit',
       ilab=signif(ator80$ldlPcfb.pbo,2),
       ilab.xpos=-90,
       ylim=c(-4,nrow(ator80)+3),
       main="Difference from Placebo in %CFB LDL")
text(-120, 7.5, "Study", cex=0.95, adj=0, font=2)
text(-95, 7.75, "Placebo\n%CFB LDL", cex=0.95, adj=0, font=2)
text(-30, 7.5, "Mean diff. + 95%CI",cex=0.95, adj=0, font=2)
addpoly(x=predVals$pred, sei=predVals$se,col='red',  cex=0.8,
        mlab=paste("Placebo response =",c(-2,0,2,4),"%"))


# 6. Try fitting one of your meta-regression models using WinBUGS. 
# How do the posterior mean and standard deviations compare to maximum 
# likelihood estimates?

mymodel <- function() {
  for (i in 1:nStudies) {
    thetahat[i] ~ dnorm(meantheta[i], precThetaHat[i])
    meantheta[i] <- alpha[i] + beta*(x[i]-xbar)
    alpha[i] ~ dnorm(meanAlpha, prec)
    precThetaHat[i] <- pow(s[i],-2)
  }
  meanAlpha ~ dnorm(0,1E-04)
  beta ~ dnorm(0, 1E-04)
  tau ~ dunif(0,100)
  prec <- pow(tau,-2)
}

bugsdata <- with(ator80, 
                 list(thetahat=diff, s=seDiff, 
                      x = ldlPcfb.pbo,
                      xbar=mean(ldlPcfb.pbo),
                      nStudies=length(trial))
                 )

bugsParameters <- c("meanAlpha","tau", "beta", "meantheta")

bugsInits <- function() {
  return(list(meanAlpha=rnorm(1, -20, 20), beta=rnorm(1,0,1),
              tau=runif(1,0,5))) 
}

reBugs1 <- bugs(data=bugsdata, inits=bugsInits, 
               parameters.to.save=bugsParameters,
               model=mymodel,
               n.chains=3, n.iter=30000, n.burnin=5000, n.thin=1,
               debug=TRUE)

# look at convergence diagnostics
plot(reBugs1)
print(reBugs1)
# convert bugs output to coda format
reBugscoda <- as.mcmc.bugs(reBugs1)
plot(reBugscoda)

# Compare estimates
coef(fit4)
c(mean=mean(reBugs1$sims.list$meanAlpha), 
  quantile(reBugs1$sims.list$meanAlpha,prob=c(0.025,.5,.975)))
c(mean=mean(reBugs1$sims.list$beta), 
  quantile(reBugs1$sims.list$beta,prob=c(0.025,.5,.975)))
plot(density(reBugs1$sims.list$beta))

# Sensitivity to the choice of prior?