modelName <- "statinHandsOn" # root names of model file

if(.Platform$OS.type == "windows") Sys.setenv(HOME = substr(R.home(), 1, 2))
courseDir <- file.path(Sys.getenv("HOME"), "MI260")
##courseDir <- file.path(Sys.getenv("HOME"), "project/MI260Summer2011")
exampleDir <- file.path(courseDir, modelName)
toolsDir <- file.path(courseDir, "bugsTools") # change to match your setup

setwd(exampleDir)
library(lattice)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

## Get data files
ldlData <- read.csv(file.path(exampleDir, "ldlDataAll.csv"), as.is = TRUE)

## Rename 0 dose treatments as "Placebo"
ldlData$drug1 <- ifelse(ldlData$dose1 == 0, "Placebo", ldlData$drug1)
ldlData$drug2 <- ifelse(ldlData$dose2 == 0, "Placebo", ldlData$drug2)

trials2Keep <- setdiff(unique(ldlData$trial[ldlData$drug1 %in%
                                            c("Atorvastatin", "Rosuvastatin")]),
                       unique(ldlData$trial[ldlData$dose2 > 0]))
ldlData <- ldlData[ldlData$trial %in% trials2Keep,]
ldlData <- ldlData[ldlData$drug1 %in% c("Atorvastatin", "Rosuvastatin", "Placebo"),]

ldlData <- ldlData[c("trial", "drug1", "dose1", "n", "week", "ldlPcfb", "ldl0",
                    "seLdl0", "seLdlPcfb", "ref")]
names(ldlData)[names(ldlData) == "drug1"] <- "drug"
names(ldlData)[names(ldlData) == "dose1"] <- "dose"

write.csv(ldlData, "ldlData.csv", row.names = FALSE)

## open graphics device
pdf(file = file.path(exampleDir, paste(modelName,"EDAPlots%03d.pdf", sep = "")),
	width = 6, height = 6, onefile = F)

## Plots for statin monotherapy
drugs <- sort(unique(ldlData$drug))
drugs <- drugs[drugs != "Placebo"]
for(drug in drugs){
  trials <- unique(ldlData$trial[ldlData$drug == drug])
  yplot <- ldlData[(ldlData$trial %in% trials) & (ldlData$drug %in% c(drug, "Placebo")),]

  print(xyplot(ldlPcfb ~ dose | as.factor(trial), yplot, groups = trial, panel =
               function(x, y, subscripts, ...){
                 panel.superpose(x, y, subscripts, ...)
                 panel.text(x, y + ifelse(x == 0, -5, 5),
                            labels = yplot$n[subscripts])
               },
               type = "b", lwd = 2, col = "blue", cex=1, scales=list(cex=1),
               xlab = list(label="dose (mg/d)", cex=1.2),
               ylab=list(label = "LDL change from baseline", cex=1.2),
               par.strip.text=list(cex=1),
               strip = function(...) strip.default(..., style = 1),
               main = list(label = drug, cex=1.5)))

}     

dev.off()
