
## Template for statinHandsOn simulations
## Replace "???????" with appropriate R code. 

modelName <- "statinReparHandsOn" # root names of model file
exampleName <- "statinSimHandsOn"

if(.Platform$OS.type == "windows") Sys.setenv(HOME = substr(R.home(), 1, 2))
courseDir <- file.path(Sys.getenv("HOME"), "MI260")
modelDir <- file.path(courseDir, modelName, modelName)
exampleDir <- file.path(courseDir, exampleName)
toolsDir <- file.path(courseDir, "bugsTools") # change to match your setup

setwd(exampleDir)
library(R2WinBUGS)
library(coda)
library(lattice)
source(file.path(toolsDir, "bugs.tools.R"))
source(file.path(toolsDir, "bgillespie.utilities.R"))

statinPopMean <- function(p, design, interTrial = TRUE){
  ## Simulate population mean for each row of design
  
  ## Arguments
  ##   p = vector of model parameters, typically one row from a matrix of posterior samples
  ##   design = data frame containing independent variable values, e.g., dose, drug and any
  ##            other covariates required by the model
  ##   interTrial = logical indicating whether inter-trial variation is treated as a component
  ##                of the posterior uncertainty. Otherwise the population mean averages over
  ##                inter-trial variation

?????????
  
}

## Specify parameters to keep
parametersToPlot <- c(
                      "e0Mean", "emaxStatin", "ed50Statin", "nStatin",
                      "omegaE0", "sigma"
                      )

################################################################################################

load(file.path(modelDir,paste(modelName,".fit.Rsave",sep="")))

## rename and reformat MCMC results to facilitate later calculations and plots
sims.array = bugs.fit$sims.array
posterior = array(as.vector(sims.array),dim=c(prod(dim(sims.array)[1:2]),dim(sims.array)[3]),
  dimnames=list(NULL,dimnames(sims.array)[[3]]))
posterior <- posterior[,unlist(sapply(c(paste("^",parametersToPlot,"$",sep=""),
	paste("^",parametersToPlot,"\\[",sep="")),grep,x=dimnames(posterior)[[2]]))]

################################################################################################

dir.create(file.path(exampleDir, exampleName))
pdf(file = file.path(exampleDir, paste(exampleName,"/",exampleName,"Plots%03d.pdf", sep = "")),
	width = 6, height = 6, onefile = F)

## Key for drug codes
## statins: atorvastatin = 1, rosuvastatin = 2

nSim <- 1000 # Number of posterior samples

## Set up design data frame
???????????????

## Statin dose-response -- averaging over inter-trial variation

## Simulate nSim posterior samples of the population mean for each row of design
pred <- ???????????????

## Calculate median and 90% prediction interval
design1 <- rbind(design, design, design)
design1$type = rep(c("5%ile","median","95%ile"), ea=nrow(pred))
design1$pcldl <- as.vector(t(apply(pred, 1, quantile, probs = c(0.05, 0.5, 0.95))))

## Replace drug with drug names
design1$drugName <- swap(design1$drug, 1:2,
                         c("atorvastatin", "rosuvastatin"))

## Simple curves
xyplot(pcldl ~ dose | as.factor(drugName), design1, groups = type,
       type = "l", lwd = 2, col = c("blue", "blue", "red"), lty = c(2, 2, 1),
       xlab=list(label = "dose (mg/d)", cex=1.2),
       ylab=list(label = "LDL % change from baseline", cex=1.2),
       scales=list(cex=1.2),
       par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1),
       sub = "averaged over inter-trial variation")

## Filled credible intervals
xyplot(pcldl ~ dose | as.factor(drugName), design1, groups = type, panel =
       function(x, y, subscripts, groups, ...){
         iMed <- (groups[subscripts] == "median")
         i5 <- (groups[subscripts] == "5%ile")
         i95 <- (groups[subscripts] == "95%ile")
         panel.polygon(c(x[i5], rev(x[i95])), c(y[i5], rev(y[i95])),
                       border = FALSE, col = "blue", alpha = 0.4, ...)
         panel.xyplot(x[iMed], y[iMed], type = "l", lwd = 2, col = "blue", ...)
       },
       xlab=list(label = "dose (mg/d)", cex=1.2),
       ylab=list(label = "LDL % change from baseline", cex=1.2),
       scales=list(cex=1.2),
       par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1),
       sub = "averaged over inter-trial variation")

## Statin dose-response -- inter-trial variation included in uncertainty bounds

## Simulate nSim posterior samples of the population mean for each row of design
pred <- ????????????????????????

## Calculate median and 90% prediction interval
design1 <- rbind(design, design, design)
design1$type = rep(c("5%ile","median","95%ile"), ea=nrow(pred))
design1$pcldl <- as.vector(t(apply(pred, 1, quantile, probs = c(0.05, 0.5, 0.95))))

## Replace drug with drug names
design1$drugName <- swap(design1$drug, 1:2,
                         c("atorvastatin", "rosuvastatin"))

## Filled credible intervals
xyplot(pcldl ~ dose | as.factor(drugName), design1, groups = type, panel =
       function(x, y, subscripts, groups, ...){
         iMed <- (groups[subscripts] == "median")
         i5 <- (groups[subscripts] == "5%ile")
         i95 <- (groups[subscripts] == "95%ile")
         panel.polygon(c(x[i5], rev(x[i95])), c(y[i5], rev(y[i95])),
                       border = FALSE, col = "blue", alpha = 0.4, ...)
         panel.xyplot(x[iMed], y[iMed], type = "l", lwd = 2, col = "blue", ...)
       },
       xlab=list(label = "dose (mg/d)", cex=1.2),
       ylab=list(label = "LDL % change from baseline", cex=1.2),
       scales=list(cex=1.2),
       par.strip.text=list(cex=1),
       strip = function(...) strip.default(..., style = 1),
       sub = "inter-trial variation included in uncertainty")

dev.off()
